/**
 * File I/O.
 * 
 * @author    Andy Frank
 * @creation  27 Dec 04
 */
 
////////////////////////////////////////////////////////////////
// Profile
////////////////////////////////////////////////////////////////

/**
 * Get the newsfox directory under the user profile
 * directory.  Create newsfox directory if it does
 * not exist. 
 */
function getProfileDir()
{
  var file = Components.classes["@mozilla.org/file/directory_service;1"].
    getService(Components.interfaces.nsIProperties).
    get("ProfD", Components.interfaces.nsIFile);      
  file.append("newsfox");
  if (!file.exists()) file.create(0x01, 0664);
  return file;
}

////////////////////////////////////////////////////////////////
// File Util
////////////////////////////////////////////////////////////////

function openOutputStream(file, flags)
{
  var out = Components.classes["@mozilla.org/network/file-output-stream;1"]
    .createInstance(Components.interfaces.nsIFileOutputStream);      
  out.init(file, flags, 0664, 0);
  return out;
}

function print(out, data)
{
  out.write(data, data.length);  
}

function println(out, data)
{
  data += "\n";
  out.write(data, data.length);
}

////////////////////////////////////////////////////////////////
// Create new feed
////////////////////////////////////////////////////////////////

function createNewFeed(model, url)
{
  try
  {  
    var uid  = model.makeUniqueUid(url); //makeUid(model, url);    
    var feed = new Feed();
    feed.uid = uid;
    feed.url = url;
    feed.defaultName = uid;
    model.add(feed);
  }
  catch (err) 
  { 
    var msg = "createNewFeed(): [" + uid + "] " + err
    alert(msg); 
  }  
}

//function makeUid(url)
//{
//  var domain = parseDomainName(url);
//  var name = domain;
//  var file = getProfileDir();
//  file.append(name + ".xml");
//
//  
//
// Can't use filenames in case file 
// hasn't been created yet.
//  for (var i=1; file.exists(); i++)
//  {
//    name = domain + i;
//    file = getProfileDir();
//    file.append(name + ".xml");
//  }  
//  
//  return name;
//}
//
//function parseDomainName(url)
//{
//  var index  = url.indexOf("://");
//  var body   = url.substring(index+3);
//  var domain = body.split("/")[0];
//  return domain;
//}

////////////////////////////////////////////////////////////////
// Load feeds from disk
////////////////////////////////////////////////////////////////

function loadFeedModel()
{
  model = new Model();
  try
  {
    var file = getProfileDir();
    file.append("master.xml");
    
    var xml = document.implementation.createDocument("","",null);
    xml.async = false;
    xml.load("file:///" + file.path);

    var kids = xml.getElementsByTagName("feed");
    for (var i=0; i<kids.length; i++)
    {
      var feed = new Feed();
      feed.uid = child(kids[i], "uid").childNodes[0].nodeValue;
      feed.url = decodeHTML(child(kids[i], "url").childNodes[0].nodeValue);
      feed.defaultName = feed.uid; 
      var v = child(kids[i], "dname");
      if (v != null) feed.defaultName = decodeHTML(v.childNodes[0].nodeValue);

      var v = child(kids[i], "home");
      if (v != null) feed.homepage = decodeHTML(v.childNodes[0].nodeValue);
      
      var v = child(kids[i], "style");
      if (v != null) feed.style = parseInt(v.childNodes[0].nodeValue);
      
      var elem = child(kids[i], "flags");
      if (elem != null && elem.childNodes.length > 0)
      {
        var flags = elem.childNodes[0].nodeValue;
        for (var j=0; j<flags.length; j++)
          feed.flags.push(parseInt(flags.charAt(j)));
      }
      
      model.add(feed);
    }
  }
  catch (err) 
  { 
    var msg = "loadFeedModel(): " + "[" + feed.uid + "," + elem + "]\n" + err; 
    alert(msg);    
  }
}

function loadFeed(feed)
{
  try
  {
    var file = getProfileDir();
    file.append(feed.uid + ".xml");
    checkFeedFile(file);
    
    var xml = document.implementation.createDocument("","",null);
    xml.async = false;
    xml.load("file:///" + file.path);
        
    var kids = xml.getElementsByTagName("article");
    for (var i=0; i<kids.length; i++)
    {
      var _link  = child(kids[i], "link").childNodes[0];
      var _title = child(kids[i], "title").childNodes[0];
      var _date  = child(kids[i], "date").childNodes[0];
      var _body  = child(kids[i], "body").childNodes[0];
      
      var art = new Article();
      art.link  = (_link  == null) ? "" : decodeHTML(_link.nodeValue);
      art.title = (_title == null) ? "" : decodeHTML(_title.nodeValue);
      art.date  = new Date();
      if (_date != null) art.date.setTime(Date.parse(_date.nodeValue));
      art.body  = (_body == null) ? "" : decodeHTML(_body.nodeValue);
      feed.addExisiting(art);
    }   
  }
  catch (err) { alert("loadFeed(): " + feed.uid + "\n" + err); }
}

function checkFeedFile(file)
{
  try
  {  
    if (!file.exists())
    {
      var out = openOutputStream(file, 0x02 | 0x08 | 0x20);
      println(out, "<newsfox-feed version=\"1.0\">");
      println(out, "</newsfox-feed>");
      out.close();
    }
  }
  catch (err) { alert(err); }  
}


////////////////////////////////////////////////////////////////
// Save feeds to disk
////////////////////////////////////////////////////////////////

function saveFeedModel()
{
  try
  {
    var file = getProfileDir();
    file.append("master.xml");
    
    var out = openOutputStream(file, 0x02 | 0x08 | 0x20);
    println(out, "<newsfox-feedlist version=\"1.0\">");
    for (var i=0; i<model.size(); i++)
    {
      var feed = model.get(i);
      println(out, " <feed>");
      println(out, "  <uid>" + feed.uid + "</uid>");
      println(out, "  <url>" + encodeHTML(feed.url) + "</url>");    
      println(out, "  <dname>" + encodeHTML(feed.defaultName) + "</dname>");
      if (feed.homepage != null)
        println(out, "  <home>" + encodeHTML(feed.homepage) + "</home>");
      // Only save if property overrides global setting
      if (feed.style != 0)
        println(out, "  <style>" + feed.style + "</style>");
      print(out, "  <flags>");
      for (var j=0; j<feed.flags.length; j++)
        print(out, "" + feed.flags[j]);
      println(out, "</flags>");
      println(out, " </feed>");      
    }
    println(out, "</newsfox-feedlist>");          
    out.close();
  }
  catch (err) { alert("saveFeedModel(): [" + feed.uid + "] " + err); }
}

function saveFeed(feed)
{
  var file = getProfileDir();
  file.append(feed.uid + ".xml");
  
  var out = openOutputStream(file, 0x02 | 0x08 | 0x20);
  println(out, "<newsfox-feed version=\"1.0\">");
  for (var i=0; i<feed.size(); i++)
  {
    var art = feed.get(i);
    println(out, " <article>");
    println(out, "  <link>" + encodeHTML(art.link) + "</link>");
    println(out, "  <title>" + encodeHTML(art.title) + "</title>");
    println(out, "  <date>" + art.date + "</date>");
    println(out, "  <body>" + encodeHTML(art.body) + "</body>");
    println(out, " </article>");
  }      
  println(out, "</newsfox-feed>");    
  out.close();    
}

////////////////////////////////////////////////////////////////
// Delete feed
////////////////////////////////////////////////////////////////

function deleteFeedFromDisk(feed)
{
  try
  {
    var file = getProfileDir();
    file.append(feed.uid + ".xml");
    if (file.exists()) file.remove(false);
  }
  catch (err) { alert("deleteFeedFromDisk(): [" + feed.uid + "] " + err); }
}

////////////////////////////////////////////////////////////////
// Util
////////////////////////////////////////////////////////////////

function child(element, tagName)
{                                          
  var kids = element.childNodes;
  for (var i=0; i<kids.length; i++)
    if (kids[i].nodeName == tagName) 
      return kids[i];
  return null;
}

////////////////////////////////////////////////////////////////
// Encode/Decode
////////////////////////////////////////////////////////////////

/**
 * Return XML-friendly HTML encoding.
 */
function encodeHTML(s) 
{
  // TODO - handle unicode
  for (var i=0; i<s.length; i++)
  {
    // TODO - this is slow - make it faster
    var code = s.charCodeAt(i);
    if (code < 32 || code > 126) 
      s = s.substring(0, i) + "?" + s.substring(i+1);
  }
  
  s = s.replace(new RegExp('&','gi'), '&amp;');
  s = s.replace(new RegExp('<','gi'), '&lt;');
  s = s.replace(new RegExp('>','gi'), '&gt;');
  s = s.replace(new RegExp('"','gi'), '&quot;');
  return s;                                              
}

/**
 * Return original HTML from encoding.
 */
function decodeHTML(s)
{
  s = s.replace(new RegExp('&amp;'  ,'gi'), '&');
  s = s.replace(new RegExp('&lt;'   ,'gi'), '<');
  s = s.replace(new RegExp('&gt;'   ,'gi'), '>');
  s = s.replace(new RegExp('&quot;' ,'gi'), '"');
  s = s.replace(new RegExp('&acute;','gi'), '');
  return s;
}

